// -*- C++ -*-
/*!
 * @file  RobotSensorDataManager.cpp
 * @brief sensor data management
 * @date $Date$
 *
 * @author East Japan Accounting Center Co.,Ltd.
 *
 * (c)2017 The University of Aizu
 * This software is released under the New BSD License.
 *
 * $Id$
 */

#include "RobotSensorDataManager.h"

// Module specification
// <rtc-template block="module_spec">
static const char* robotsensordatamanager_spec[] =
  {
    "implementation_id", "RobotSensorDataManager",
    "type_name",         "RobotSensorDataManager",
    "description",       "sensor data management",
    "version",           "0.1",
    "vendor",            "EAC",
    "category",          "sensor",
    "activity_type",     "PERIODIC",
    "kind",              "DataFlowComponent",
    "max_instance",      "1",
    "language",          "C++",
    "lang_type",         "compile",
    // Configuration variables
    "conf.default.Output_Cycle", "0",
    // Widget
    "conf.__widget__.Output_Cycle", "spin.500",
    // Constraints
    "conf.__constraints__.Output_Cycle", "0<=x<=10000",
    ""
  };
// </rtc-template>

/*!
 * @brief constructor
 * @param manager Maneger Object
 */
RobotSensorDataManager::RobotSensorDataManager(RTC::Manager* manager)
    // <rtc-template block="initializer">
  : RTC::DataFlowComponentBase(manager),
    m_inSpiderSensorDataIn("inSpiderSensorData", m_inSpiderSensorData),
    m_inDroneSensorDataIn("inDroneSensorData", m_inDroneSensorData),
    m_inGoroneSensorDataIn("inGoroneSensorData", m_inGoroneSensorData),
    m_outRobotSensorDataOut("outRobotSensorData", m_outRobotSensorData),
    m_SensorDataManagerReqServPort("SensorDataManagerReqServ")

    // </rtc-template>
{

  RobotSensorData_p[SENSOR_ID_SPIDER] = &m_stSpiderSensorData;
  RobotSensorData_p[SENSOR_ID_DRONE_1] = &m_stDrone1SensorData;
  RobotSensorData_p[SENSOR_ID_DRONE_2] = &m_stDrone2SensorData;
  RobotSensorData_p[SENSOR_ID_DRONE_3] = &m_stDrone3SensorData;
  RobotSensorData_p[SENSOR_ID_GORONE_1] = &m_stGorone1SensorData;
  RobotSensorData_p[SENSOR_ID_GORONE_2] = &m_stGorone2SensorData;
  RobotSensorData_p[SENSOR_ID_GORONE_3] = &m_stGorone3SensorData;
  RobotSensorData_p[SENSOR_ID_GORONE_4] = &m_stGorone4SensorData;
 
}

/*!
 * @brief destructor
 */
RobotSensorDataManager::~RobotSensorDataManager()
{
}


/*!
 * ロボットセンサデータ管理RTCの初期化処理
 */
RTC::ReturnCode_t RobotSensorDataManager::onInitialize()
{
  // Registration: InPort/OutPort/Service
  // <rtc-template block="registration">
  // Set InPort buffers
  addInPort("inSpiderSensorData", m_inSpiderSensorDataIn);
  addInPort("inDroneSensorData", m_inDroneSensorDataIn);
  addInPort("inGoroneSensorData", m_inGoroneSensorDataIn);
  
  // Set OutPort buffer
  addOutPort("outRobotSensorData", m_outRobotSensorDataOut);
  
  // Set service provider to Ports
  m_SensorDataManagerReqServPort.registerProvider("SensorDataOutputSelect", "SensorDataManagerReqServ::SensorDataOutputSelect", m_SensorDataOutputSelectProv);
  
  // Set service consumers to Ports
  
  // Set CORBA Service Ports
  addPort(m_SensorDataManagerReqServPort);
  
  // </rtc-template>

  // <rtc-template block="bind_config">
  // Bind variables and configuration variable
  bindParameter("Output_Cycle", m_Output_Cycle, "0");
  // </rtc-template>

#ifdef _DEBUG
  std::cout << "[info]RobotSensorDataManager::onInitialize()" << std::endl;
#endif

  m_inSpiderSensorData.data.length(SPIDER_CONNECT_MAX);
  m_inDroneSensorData.data.length(DRONE_CONNECT_MAX);
  m_inGoroneSensorData.data.length(GORONE_CONNECT_MAX);

  nvdRobotSensorDataInit();

  return RTC::RTC_OK;
}

/*!
 * ロボットセンサデータ管理RTCの終了処理
 */

RTC::ReturnCode_t RobotSensorDataManager::onFinalize()
{
#ifdef _DEBUG
	std::cout << "[info]RobotSensorDataManager::onFinalize()" << std::endl;
#endif

	return RTC::RTC_OK;
}

/*
RTC::ReturnCode_t RobotSensorDataManager::onStartup(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t RobotSensorDataManager::onShutdown(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*!
 * ロボットセンサデータ管理RTCの開始処理
 */

RTC::ReturnCode_t RobotSensorDataManager::onActivated(RTC::UniqueId ec_id)
{
#ifdef _DEBUG
	std::cout << "[info]RobotSensorDataManager::onActivated()" << std::endl;
#endif

	//入力データポート初期化
	while(m_inSpiderSensorDataIn.isEmpty() == FALSE){
		m_inSpiderSensorDataIn.read();
	}

	while(m_inDroneSensorDataIn.isEmpty() == FALSE){
		m_inDroneSensorDataIn.read();
	}

	while(m_inGoroneSensorDataIn.isEmpty() == FALSE){
		m_inGoroneSensorDataIn.read();
	}

	//経過時間保持変数を初期化
	m_LastTime = std::chrono::system_clock::now();

	return RTC::RTC_OK;
}

/*!
 * ロボットセンサデータ管理RTCの停止処理
 */

RTC::ReturnCode_t RobotSensorDataManager::onDeactivated(RTC::UniqueId ec_id)
{
#ifdef _DEBUG
	std::cout << "[info]RobotSensorDataManager::onDeactivated()" << std::endl;
#endif

	nvdRobotSensorDataInit();

	return RTC::RTC_OK;
}

/*!
 * ロボットセンサデータ管理RTCの周期処理
 */

RTC::ReturnCode_t RobotSensorDataManager::onExecute(RTC::UniqueId ec_id)
{
	std::chrono::system_clock::time_point  CurrentTime;
	double dElapsedTime = 0;

	CurrentTime = std::chrono::system_clock::now();	//現在の時間を取得
	dElapsedTime = std::chrono::duration_cast<std::chrono::milliseconds>(CurrentTime - m_LastTime).count();	//経過時間をミリ秒に変換

	//センサデータ読み込み+パーサー
	nvdReadSpiderSensorData();
	nvdReadDroneSensorData();
	nvdReadGoroneSensorData();
	
	//コンフィギュレーション出力周期経過後、GPSデータ出力
	if(dElapsedTime >= m_Output_Cycle){
		//センサデータ出力
		uhwRobotSensorDataOutput();
		m_LastTime = CurrentTime;
	}

	return RTC::RTC_OK;
}

/*!
 * ロボットセンサデータ管理RTCのエラー遷移処理
 */

RTC::ReturnCode_t RobotSensorDataManager::onAborting(RTC::UniqueId ec_id)
{
#ifdef _DEBUG
	std::cout << "[info]RobotSensorDataManager::onAborting()" << std::endl;
#endif

	return RTC::RTC_OK;
}

/*
RTC::ReturnCode_t RobotSensorDataManager::onError(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t RobotSensorDataManager::onReset(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t RobotSensorDataManager::onStateUpdate(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t RobotSensorDataManager::onRateChanged(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*****************************************************************************************
*	Function Name		：nvdReadSpiderSensorData
*	Description			：スパイダーセンサデータポート読み込み
*	Argument			：なし
*	Return				：なし
******************************************************************************************/
void RobotSensorDataManager::nvdReadSpiderSensorData(void)
{
	Json::Reader reader;
	Json::Value	Val;
	std::string strSensorData = "";

	if(m_inSpiderSensorDataIn.isNew()){
		m_inSpiderSensorDataIn.read();
		strSensorData = m_inSpiderSensorData.data[0];
	}

	if(strSensorData != ""){
		reader.parse(strSensorData,Val);
		m_stSpiderSensorData.strRobotName = JSONKEY_ROBOT_SPIDER;
		m_stSpiderSensorData.stSensorData.dGyro_Roll = Val[JSONKEY_SENSOR_GYRO][JSONKEY_ROLL].asDouble();
		m_stSpiderSensorData.stSensorData.dGyro_Pitch = Val[JSONKEY_SENSOR_GYRO][JSONKEY_PITCH].asDouble();
		m_stSpiderSensorData.stSensorData.dGyro_Yaw = Val[JSONKEY_SENSOR_GYRO][JSONKEY_YAW].asDouble();
		m_stSpiderSensorData.stSensorData.dSpeed_XAxis = Val[JSONKEY_SENSOR_SPEED][JSONKEY_X_AXIS].asDouble();
		m_stSpiderSensorData.stSensorData.dSpeed_YAxis = Val[JSONKEY_SENSOR_SPEED][JSONKEY_Y_AXIS].asDouble();
		m_stSpiderSensorData.stSensorData.dSpeed_Yaw = Val[JSONKEY_SENSOR_SPEED][JSONKEY_YAW].asDouble();
		m_stSpiderSensorData.stSensorData.dSpeed = 0;		//未使用データ
		m_stSpiderSensorData.stSensorData.dTemp = Val[JSONKEY_SENSOR_TEMP].asDouble();
		m_stSpiderSensorData.stSensorData.dCompass = 0;		//未使用データ
		m_stSpiderSensorData.stSensorData.dWindPress = 0;	//未使用データ
	}
}

/*****************************************************************************************
*	Function Name		：nvdReadDroneSensorData
*	Description			：ドローンセンサデータポート読み込み
*	Argument			：なし
*	Return				：なし
******************************************************************************************/
void RobotSensorDataManager::nvdReadDroneSensorData(void)
{
	Json::Reader reader;
	Json::Value	Val;
	std::string strSensorData = "";
	uhword_type uhwLoop = 0;

	std::string strSelectDroneNameTbl[DRONE_CONNECT_MAX] ={
		JSONKEY_ROBOT_DRONE_1,
		JSONKEY_ROBOT_DRONE_2,
		JSONKEY_ROBOT_DRONE_3
	};

	if(m_inDroneSensorDataIn.isNew()){
		m_inDroneSensorDataIn.read();

		for(uhwLoop = 0; uhwLoop < DRONE_CONNECT_MAX; uhwLoop ++){
			strSensorData = m_inDroneSensorData.data[uhwLoop];
			if(strSensorData != ""){
				reader.parse(strSensorData,Val);

				RobotSensorData_st	stSensorData;
				std::string strName = strSelectDroneNameTbl[uhwLoop];
				stSensorData.strRobotName = strName;
				stSensorData.stSensorData.dGyro_Roll = Val[JSONKEY_SENSOR_GYRO][JSONKEY_ROLL].asDouble();
				stSensorData.stSensorData.dGyro_Pitch = Val[JSONKEY_SENSOR_GYRO][JSONKEY_PITCH].asDouble();
				stSensorData.stSensorData.dGyro_Yaw = Val[JSONKEY_SENSOR_GYRO][JSONKEY_YAW].asDouble();
				stSensorData.stSensorData.dSpeed_XAxis = 0;	//未使用データ
				stSensorData.stSensorData.dSpeed_YAxis = 0;	//未使用データ
				stSensorData.stSensorData.dSpeed_Yaw = 0;	//未使用データ
				stSensorData.stSensorData.dSpeed = Val[JSONKEY_SENSOR_SPEED].asDouble();
				stSensorData.stSensorData.dTemp = Val[JSONKEY_SENSOR_TEMP].asDouble();
				stSensorData.stSensorData.dCompass = Val[JSONKEY_SENSOR_COMPASS].asDouble();
				stSensorData.stSensorData.dWindPress = Val[JSONKEY_SENSOR_WINDPRESS].asDouble();

				nvdSetSelectRobotData((uhwLoop + SENSOR_ID_DRONE_1), stSensorData);
			}
			else{
				std::cout << "[warrning]Drone sensor data is empty = " << strSelectDroneNameTbl[uhwLoop] << std::endl;
			}
		}
	}
}

/*****************************************************************************************
*	Function Name		：nvdReadGoroneSensorData
*	Description			：ゴローンセンサデータポート読み込み
*	Argument			：なし
*	Return				：なし
******************************************************************************************/
void RobotSensorDataManager::nvdReadGoroneSensorData(void)
{
	Json::Reader reader;
	Json::Value	Val;
	std::string strSensorData = "";
	uhword_type uhwLoop = 0;

	std::string strSelectGoroneNameTbl[GORONE_CONNECT_MAX] ={
		JSONKEY_ROBOT_GORONE_1,
		JSONKEY_ROBOT_GORONE_2,
		JSONKEY_ROBOT_GORONE_3,
		JSONKEY_ROBOT_GORONE_4,
	};

	if(m_inGoroneSensorDataIn.isNew()){
		m_inGoroneSensorDataIn.read();

		for(uhwLoop = 0; uhwLoop < GORONE_CONNECT_MAX; uhwLoop ++){
			strSensorData = m_inGoroneSensorData.data[uhwLoop];

			if(strSensorData != ""){
				reader.parse(strSensorData,Val);

				RobotSensorData_st	stSensorData;
				std::string strName = strSelectGoroneNameTbl[uhwLoop];
				stSensorData.strRobotName = strName;
				stSensorData.stSensorData.dGyro_Roll = Val[JSONKEY_SENSOR_GYRO][JSONKEY_ROLL].asDouble();
				stSensorData.stSensorData.dGyro_Pitch = Val[JSONKEY_SENSOR_GYRO][JSONKEY_PITCH].asDouble();
				stSensorData.stSensorData.dGyro_Yaw = Val[JSONKEY_SENSOR_GYRO][JSONKEY_YAW].asDouble();
				stSensorData.stSensorData.dSpeed_XAxis = Val[JSONKEY_SENSOR_SPEED][JSONKEY_X_AXIS].asDouble();
				stSensorData.stSensorData.dSpeed_YAxis = Val[JSONKEY_SENSOR_SPEED][JSONKEY_Y_AXIS].asDouble();
				stSensorData.stSensorData.dSpeed_Yaw = Val[JSONKEY_SENSOR_SPEED][JSONKEY_YAW].asDouble();
				stSensorData.stSensorData.dSpeed = 0;		//未使用データ
				stSensorData.stSensorData.dTemp = 0;		//未使用データ
				stSensorData.stSensorData.dCompass = Val[JSONKEY_SENSOR_COMPASS].asDouble();
				stSensorData.stSensorData.dWindPress = 0;	//未使用データ

				nvdSetSelectRobotData((uhwLoop + SENSOR_ID_GORONE_1), stSensorData);
			}
			else{
				std::cout << "[warrning]Gorone sensor data is empty = " << strSelectGoroneNameTbl[uhwLoop] << std::endl;
			}
		}
	}
}

/*****************************************************************************************
*	Function Name		：uhwRobotSensorDataOutput
*	Description			：指定ロボットセンサデータ出力
*	Argument			：なし
*	Return				：正常終了：RESULT_SUCCESS、異常終了：RESULT_WARNING
******************************************************************************************/
uhword_type RobotSensorDataManager::uhwRobotSensorDataOutput(void)
{
	Json::StyledWriter	writer;
	std::string strJson = "";

	nvdRobotSensorDataReqSet();

	if(m_uwCurrentOutputSel >= ROBOT_SEL_SPIDER){
		//JSON格納変数の初期化
		nvdJsonDataInit();

		uhword_type uhwTblNo = m_uwCurrentOutputSel - ROBOT_SEL_SPIDER;

		m_JsonData[JSONKEY_ROBOT_NAME] = RobotSensorData_p[uhwTblNo]->strRobotName;
		m_JsonData[JSONKEY_SENSOR_GYRO][JSONKEY_ROLL] = RobotSensorData_p[uhwTblNo]->stSensorData.dGyro_Roll;
		m_JsonData[JSONKEY_SENSOR_GYRO][JSONKEY_PITCH] = RobotSensorData_p[uhwTblNo]->stSensorData.dGyro_Pitch;
		m_JsonData[JSONKEY_SENSOR_GYRO][JSONKEY_YAW] = RobotSensorData_p[uhwTblNo]->stSensorData.dGyro_Yaw;
		m_JsonData[JSONKEY_SENSOR_SPEED][JSONKEY_ROBOTTYPE_CRAWLER][JSONKEY_X_AXIS] = RobotSensorData_p[uhwTblNo]->stSensorData.dSpeed_XAxis;
		m_JsonData[JSONKEY_SENSOR_SPEED][JSONKEY_ROBOTTYPE_CRAWLER][JSONKEY_Y_AXIS] = RobotSensorData_p[uhwTblNo]->stSensorData.dSpeed_YAxis;
		m_JsonData[JSONKEY_SENSOR_SPEED][JSONKEY_ROBOTTYPE_CRAWLER][JSONKEY_YAW] = RobotSensorData_p[uhwTblNo]->stSensorData.dSpeed_Yaw;
		m_JsonData[JSONKEY_SENSOR_SPEED][JSONKEY_ROBOTTYPE_DRONE] = RobotSensorData_p[uhwTblNo]->stSensorData.dSpeed;
		m_JsonData[JSONKEY_SENSOR_TEMP] = RobotSensorData_p[uhwTblNo]->stSensorData.dTemp;
		m_JsonData[JSONKEY_SENSOR_COMPASS] = RobotSensorData_p[uhwTblNo]->stSensorData.dCompass;
		m_JsonData[JSONKEY_SENSOR_WINDPRESS] = RobotSensorData_p[uhwTblNo]->stSensorData.dWindPress;

		//jsonフォーマットに変換し出力
		strJson = writer.write(m_JsonData);

		std::cout << "[debug]output = " << strJson << std::endl;

		m_outRobotSensorData.data = strJson.c_str();
		m_outRobotSensorDataOut.write();
	}

	return RESULT_SUCCESS;
}

/*****************************************************************************************
*	Function Name		：nvdRobotSensorDataReqSet
*	Description			：ロボットセンサデータ出力要求設定
*	Argument			：なし
*	Return				：なし
******************************************************************************************/
void RobotSensorDataManager::nvdRobotSensorDataReqSet(void)
{
	uword_type uwOutSel = ROBOT_SEL_OFF;

	m_SensorDataOutputSelectProv.GetSensorDataOutputSelect(uwOutSel);

	if(uwOutSel >= ROBOT_SEL_MAX){
		std::cout << "[warrning]GetSensorDataOutputSelect is range out!!" << std::endl;
		return;
	}

	if(uwOutSel != m_uwCurrentOutputSel){
		m_uwCurrentOutputSel = uwOutSel;
	}
}

/******************************************************************************************
*	Function Name		：nvdSetSelectDroneData
*	Description			：指定ドローンのセンサデータ設定処理
*	Argument			：第一：指定ドローンNo、第二：設定するセンサデータ
*	Return				：なし
******************************************************************************************/
void RobotSensorDataManager::nvdSetSelectRobotData(uhword_type uhwSelectNo,RobotSensorData_st stRobotData)
{
	if(uhwSelectNo < SENSOR_SUPPORT_ROBOT_MAX){
		RobotSensorData_p[uhwSelectNo]->strRobotName = stRobotData.strRobotName;
		RobotSensorData_p[uhwSelectNo]->stSensorData.dGyro_Roll = stRobotData.stSensorData.dGyro_Roll;
		RobotSensorData_p[uhwSelectNo]->stSensorData.dGyro_Pitch = stRobotData.stSensorData.dGyro_Pitch;
		RobotSensorData_p[uhwSelectNo]->stSensorData.dGyro_Yaw = stRobotData.stSensorData.dGyro_Yaw;
		RobotSensorData_p[uhwSelectNo]->stSensorData.dSpeed_XAxis = stRobotData.stSensorData.dSpeed_XAxis;
		RobotSensorData_p[uhwSelectNo]->stSensorData.dSpeed_YAxis = stRobotData.stSensorData.dSpeed_YAxis;
		RobotSensorData_p[uhwSelectNo]->stSensorData.dSpeed_Yaw = stRobotData.stSensorData.dSpeed_Yaw;
		RobotSensorData_p[uhwSelectNo]->stSensorData.dSpeed = stRobotData.stSensorData.dSpeed;
		RobotSensorData_p[uhwSelectNo]->stSensorData.dTemp = stRobotData.stSensorData.dTemp;
		RobotSensorData_p[uhwSelectNo]->stSensorData.dCompass = stRobotData.stSensorData.dCompass;
		RobotSensorData_p[uhwSelectNo]->stSensorData.dWindPress = stRobotData.stSensorData.dWindPress;
	}
	else{
		std::cout << "[debug]nvdSetSelectRobotData()parameter is out of range!!" << std::endl;
	}
}

/******************************************************************************************
*	Function Name		：nvdJsonDataInit
*	Description			：Jsonデータ格納変数の初期化
*	Argument			：なし
*	Return				：なし
******************************************************************************************/
void RobotSensorDataManager::nvdJsonDataInit(void)
{
	//Jsonデータクリア
	m_JsonData.clear();

	m_JsonData[JSONKEY_ROBOT_NAME] = "";
	m_JsonData[JSONKEY_SENSOR_GYRO][JSONKEY_ROLL] = 0;
	m_JsonData[JSONKEY_SENSOR_GYRO][JSONKEY_PITCH] = 0;
	m_JsonData[JSONKEY_SENSOR_GYRO][JSONKEY_YAW] = 0;
	m_JsonData[JSONKEY_SENSOR_SPEED][JSONKEY_ROBOTTYPE_CRAWLER][JSONKEY_X_AXIS] = 0;
	m_JsonData[JSONKEY_SENSOR_SPEED][JSONKEY_ROBOTTYPE_CRAWLER][JSONKEY_Y_AXIS] = 0;
	m_JsonData[JSONKEY_SENSOR_SPEED][JSONKEY_ROBOTTYPE_CRAWLER][JSONKEY_YAW] = 0;
	m_JsonData[JSONKEY_SENSOR_SPEED][JSONKEY_ROBOTTYPE_DRONE] = 0;
	m_JsonData[JSONKEY_SENSOR_TEMP] = 0;
	m_JsonData[JSONKEY_SENSOR_COMPASS] = 0;
	m_JsonData[JSONKEY_SENSOR_WINDPRESS] = 0;
}

/******************************************************************************************
*	Function Name		：nvdRobotSensorDataInit
*	Description			：変数初期化
*	Argument			：なし
*	Return				：なし
******************************************************************************************/
void RobotSensorDataManager::nvdRobotSensorDataInit(void)
{
	uhword_type uhwLoop = 0;

	for(uhwLoop = 0; uhwLoop < SENSOR_SUPPORT_ROBOT_MAX; uhwLoop++){
		RobotSensorData_p[uhwLoop]->strRobotName = "";
		RobotSensorData_p[uhwLoop]->stSensorData.dGyro_Roll = 0;
		RobotSensorData_p[uhwLoop]->stSensorData.dGyro_Pitch = 0;
		RobotSensorData_p[uhwLoop]->stSensorData.dGyro_Yaw = 0;
		RobotSensorData_p[uhwLoop]->stSensorData.dSpeed_XAxis = 0;
		RobotSensorData_p[uhwLoop]->stSensorData.dSpeed_YAxis = 0;
		RobotSensorData_p[uhwLoop]->stSensorData.dSpeed_Yaw = 0;
		RobotSensorData_p[uhwLoop]->stSensorData.dSpeed = 0;
		RobotSensorData_p[uhwLoop]->stSensorData.dTemp = 0;
		RobotSensorData_p[uhwLoop]->stSensorData.dCompass = 0;
		RobotSensorData_p[uhwLoop]->stSensorData.dWindPress = 0;
	}

	m_uwCurrentOutputSel = ROBOT_SEL_OFF;

	m_SensorDataOutputSelectProv.initSensorDataOutputSelect();
}

extern "C"
{
 
  void RobotSensorDataManagerInit(RTC::Manager* manager)
  {
    coil::Properties profile(robotsensordatamanager_spec);
    manager->registerFactory(profile,
                             RTC::Create<RobotSensorDataManager>,
                             RTC::Delete<RobotSensorDataManager>);
  }
  
};


